use pubs -- database to install
go

-- Setup Error Printing
-- 1. create procedure dbo.usp_PrintError
--
IF OBJECTPROPERTY(object_id(N'dbo.usp_PrintError'), 'IsProcedure') = 1
	DROP PROCEDURE dbo.usp_PrintError
go

CREATE PROCEDURE dbo.usp_PrintError
AS
BEGIN
    PRINT 'Error ' + CONVERT(nvarchar(12), ERROR_NUMBER()) +
          ', Severity ' + CONVERT(nvarchar(5), ERROR_SEVERITY()) +
          ', State ' + CONVERT(nvarchar(5), ERROR_STATE()) +
          ', Procedure ' + ISNULL(ERROR_PROCEDURE(), '-') +
          ', Line ' + CONVERT(nvarchar(5), ERROR_LINE());
    PRINT ERROR_MESSAGE();
END 
GO

-- Setup Error Logging
-- 1. create dbo.ErrorLog table
-- 2. create procedure dbo.usp_LogError
IF OBJECTPROPERTY(object_id(N'dbo.ErrorLog'), 'IsTable') = 1
	DROP TABLE dbo.ErrorLog
go

CREATE TABLE dbo.ErrorLog (
  ErrorLogID int IDENTITY 
    CONSTRAINT PK_ErrorLog_ErrorLogID PRIMARY KEY,
  ErrorTime datetime NOT NULL
    CONSTRAINT DF_ErrorLog_ErrorTime DEFAULT (GETDATE()),
  UserName sysname NOT NULL,
      ErrorNumber int NOT NULL,
      ErrorSeverity int NULL,
      ErrorState int NULL,
      ErrorProcedure nvarchar(126) NULL,
      ErrorLine int NULL,
      ErrorMessage nvarchar(4000) NOT NULL
)
GO

IF OBJECTPROPERTY(object_id(N'dbo.usp_LogError'), 'IsProcedure') = 1
	DROP PROCEDURE dbo.usp_LogError
go

CREATE PROCEDURE dbo.usp_LogError
  @ErrorLogID int = 0 OUTPUT
AS
BEGIN
  SET NOCOUNT ON;

  -- output parm 0 indicates error info not logged
  SET @ErrorLogID = 0;

  BEGIN TRY
    -- return if no error
    IF ERROR_NUMBER() IS NULL 
      RETURN;

    -- return if inside uncommitable tx
    IF XACT_STATE() = -1
    BEGIN
      PRINT N'Transaction Uncommittable. Cannot Log Error'
      RETURN
    END

    INSERT dbo.ErrorLog 
      (
      UserName,
      ErrorNumber,
      ErrorSeverity,
      ErrorState,
      ErrorProcedure,
      ErrorLine,
      ErrorMessage
      )
    VALUES
      (
      CONVERT(sysname, CURRENT_USER),
      ERROR_NUMBER(),
      ERROR_SEVERITY(),
      ERROR_STATE(),
      ERROR_PROCEDURE(),
      ERROR_LINE(),    
      ERROR_MESSAGE()
      );

    SELECT @ErrorLogID = @@IDENTITY;
  END TRY
  BEGIN CATCH
    PRINT 'An error occurred in stored procedure usp_LogError'
    EXECUTE dbo.usp_PrintError;
    RETURN -1;
  END CATCH;
END;
GO

-- Setup Error Rethrow
-- 1. create procedure dbo.usp_RethrowError

IF OBJECTPROPERTY(object_id(N'dbo.usp_RethrowError'), 'IsProcedure') = 1
	DROP PROCEDURE dbo.usp_RethrowError
go

CREATE PROCEDURE dbo.usp_RethrowError
AS
BEGIN
  -- return if not an error
  IF ERROR_NUMBER() IS NULL 
    RETURN;

  DECLARE 
    @ErrorMessage   NVARCHAR(4000), 
    @ErrorNumber    INT, 
    @ErrorSeverity  INT, 
    @ErrorState     INT, 
    @ErrorLine      INT,
    @ErrorProcedure NVARCHAR(256);    

  SELECT
    @ErrorNumber = ERROR_NUMBER(), 
    @ErrorSeverity = ERROR_SEVERITY(), 
    @ErrorState = ERROR_STATE(), 
    @ErrorLine = ERROR_LINE(),
    @ErrorProcedure = ISNULL(ERROR_PROCEDURE(), '-'); 

  -- Print error message
  SELECT @ErrorMessage = 
    N'Error %d, Level %d, State %d, Procedure %s, Line %d, ' +
    N'Message: ' + ERROR_MESSAGE();

  RAISERROR
    (
     @ErrorMessage,
     @ErrorSeverity,
     1,			-- to prevent error in raiserror
     @ErrorNumber,
     @ErrorSeverity,
     @ErrorState,
     @ErrorProcedure,
     @ErrorLine
     );
END
GO

IF OBJECTPROPERTY(object_id(N'dbo.usp_RethrowError2'), 'IsProcedure') = 1
	DROP PROCEDURE dbo.usp_RethrowError2
go

CREATE PROCEDURE dbo.usp_RethrowError2 (
  @original_procedure NVARCHAR(256),
  @original_line INT
)
AS
BEGIN
  -- return if not an error
  IF ERROR_NUMBER() IS NULL 
    RETURN;

  DECLARE 
    @ErrorMessage   NVARCHAR(4000), 
    @ErrorNumber    INT, 
    @ErrorSeverity  INT, 
    @ErrorState     INT, 
    @ErrorLine      INT,
    @ErrorProcedure NVARCHAR(256);    

  SELECT
    @ErrorNumber = ERROR_NUMBER(), 
    @ErrorSeverity = ERROR_SEVERITY(), 
    @ErrorState = ERROR_STATE(), 
--    @ErrorLine = ERROR_LINE(),
--    @ErrorProcedure = ISNULL(ERROR_PROCEDURE(), '-'); 

    @ErrorLine = @original_line,
    @ErrorProcedure = ISNULL(@original_procedure, '-'); 

  -- Print error message
  SELECT @ErrorMessage = 
    N'Error %d, Level %d, State %d, Procedure %s, Line %d, ' +
    N'Message: ' + ERROR_MESSAGE();

  RAISERROR
    (
     @ErrorMessage,
     @ErrorSeverity,
     1,			-- to prevent error in raiserror
     @ErrorNumber,
     @ErrorSeverity,
     @ErrorState,
     @ErrorProcedure,
     @ErrorLine
     );
END
GO
